'use strict';
const tredux = require('tredux'),
  config = require('config'),
  fetcher = require('thorin-fetch'),
  router = require('router'),
  qs = require('querystring'),
  util = require('lib/util');

const apiFetch = fetcher('api', {
  credentials: typeof config.credentials === 'boolean' ? config.credentials : true,
  url: config.api
});

const api = {};
api.fetcher = fetcher;


/*
* Perform API upload
* */
api.upload = function upload(url, fileObj, opt) {
  if (!opt) opt = {};
  return fetcher.upload({
    credentials: true,
    url: url,
    ...opt
  }).send(fileObj);
};

api.fetch = function fetch(action, payload, opt, _ignoreDispatch, _ignoreLoading) {
  let args = Array.prototype.slice.call(arguments),
    displayError = (payload !== false),
    doneFn = (typeof payload === 'function' ? payload : args.pop()),
    loadType = (payload === true || opt === true ? 'loading' : 'fetching'),
    wasLoading = loadType === 'loading' ? util.isLoading() : util.isFetching();
  if (payload === true && typeof opt === 'undefined' && typeof _ignoreDispatch === 'undefined' && typeof _ignoreLoading === 'undefined') {
    _ignoreDispatch = true;
  }
  if (!wasLoading && _ignoreLoading !== true) {
    util[loadType]();
  }
  let actionPromise, dispatchType;
  if (typeof action === 'object' && action) {
    payload = action.payload || {};
    actionPromise = action.promise;
    dispatchType = action.dispatch_type || action.type;
    action = action.type;
  } else {
    dispatchType = action;
  }
  if (typeof dispatchType === 'undefined') {
    return Promise.reject(new Error('Missing dispatch type in api fetch'));
  }
  let actionName = (typeof action === 'object' ? action.type : action);
  if (!payload) payload = {};
  let isFetched = false;

  // we have a promise inside, we use it.
  function doRequest(resolve, reject) {
    if (!actionPromise) {
      isFetched = true;
      actionPromise = apiFetch(actionName, payload, opt);
    }
    if (tredux.isReady() && _ignoreDispatch !== true) {
      return tredux.dispatch({
        type: dispatchType,
        promise: actionPromise,
        payload
      }).then((res) => {
        if (!wasLoading && _ignoreLoading !== true) {
          util[loadType](false);
        }
        resolve(res);
      }).catch((e) => {
        let loginUrl = getExpiredUrl(e);
        if (loginUrl) {
          window.location = loginUrl;
          return;
        }
        handleError(e);
        reject(e);
      });
    } else {
      return actionPromise.then((res) => {
        if (!wasLoading && _ignoreLoading !== true) {
          util[loadType](false);
        }
        resolve(res);
      }).catch((e) => {
        let loginUrl = getExpiredUrl(e);
        if (loginUrl) {
          window.location = loginUrl;
          return;
        }
        handleError(e);
        reject(e);
      });
    }
  }

  function handleError(e) {
    if (e && e.status === 401 && e.code === 'AUTH.EXPIRED') {
      if (e.data && e.data.url) {
        window.location.href = e.data.url;
        return;
      }
    }
    if (!wasLoading && _ignoreLoading !== true) {
      util[loadType](false);
    }
    if (e.fields instanceof Array) {
      e.data = e.fields;
    }
    if (e.data instanceof Array) {
      e.fields = [];
      for (let i = 0; i < e.data.length; i++) {
        let f = e.data[i];
        let item;
        if (typeof f === 'string') {
          item = {};
          item.field = f;
          item.message = 'Please provide this information';
        } else if (typeof f === 'object' && f) {
          item = f;
        }
        if (item) {
          e.fields.push(item);
        }
      }
    } else if (typeof e.data === 'object' && e.data) {
      e.fields = [];
      if (typeof e.data.field === 'string') {
        e.fields.push({
          field: e.data.field,
          message: e.message
        });
      }
    } else {
      e.fields = [];
    }
    if (config.debug) {
      console.warn(`[${e.code}] - ${e.message}${e.fields ? ' ' + JSON.stringify(e.fields.map(i => i.field)) : ''}`);
    }
    if (e.code !== 'FIELD.INVALID' && displayError && e.fields.length == 0) {
      util.error(e);
      console.warn(e);
    }
  }

  if (typeof doneFn !== 'function') {
    return new Promise(doRequest);
  }
  doRequest((res) => {
    doneFn(null, res);
  }, (e) => {
    doneFn(e);
  });
}

/*
* Performs generic logout
* */
api.logout = async function logout(redirect) {
  if (redirect !== false) {
    util.loading();
  }
  fetch('auth.logout')
    .catch((e) => {
      if (NODE_ENV === 'development') {
        console.warn(e);
        return;
      }
    })
    .finally(() => {
      if (redirect !== false && typeof config.login === 'string') {
        window.location = config.login;
        util.loading(false);
      }
    });
}

/*
* PRIVATE FUNCTIONALITY
* */
function getExpiredUrl(e) {
  if (!e || !e.code || e.code.indexOf('AUTH') === -1) return null;
  let data = e.data || {};
  if (!data || !data.url) {
    if (typeof config.login === 'string') return config.login;
    return null;
  }
  try {
    let urlPath = (window.location.href.split(window.location.host)[1] || '/').split('?')[0];
    let p = util.getPayload();
    delete p.token;
    delete p.continue;
    delete p.switch;
    let str = qs.stringify(p);
    if (str) urlPath += '?' + str;
    urlPath = btoa(urlPath);
    data.url += (data.url.indexOf('?') === -1 ? '?' : '&') + 'continue=' + urlPath;
  } catch (e) {
    console.log(e);
  }
  return data.url;
}

module.exports = api;
