'use strict';
const reactRouter = require('react-router');
const createMemoryHistory = require('history').createMemoryHistory;

const {
  Router,
  Link,
  IndexRedirect,
  IndexRoute,
  Redirect,
  Route,
  useRouterHistory,
  browserHistory
} = reactRouter;
const qs = require('querystring'),
  util = require('./util'),
  tredux = require('tredux'),
  React = require('react');

let PageNotFound = require('core/containers/NotFound');
export function setPageNotFound(p) {
  PageNotFound = p;
}
/**
 * React Router module router registering.
 */
const ROUTES = [],
  ROOT_ROUTES = [];
let persistentPayload = null;

function createRouteElement(RouteComponent) {
  let _routeFn = RouteComponent.createRouteFromReactElement;
  return function createRouteFromReactElement(element, parent) {
    const elem = {
      type: element.type,
      props: Object.assign({}, element.props)
    };
    let _type = typeof elem.props.component;
    if (_type === 'function' || _type === 'object') {
      let Comp = elem.props.component;
      if (typeof Comp.load === 'function') {
        elem.props.onEnter = enter(Comp, Comp.load);
        elem.props.onChange = enter(Comp, Comp.load, true);
      } else {
        if (typeof Comp.enter === 'function') {
          elem.props.onEnter = enter(Comp);
        }
        if (typeof Comp.change === 'function') {
          elem.props.onChange = change(Comp);
        }
      }
      if (typeof Comp.leave === 'function') {
        elem.props.onLeave = leave(Comp);
      }
    }
    return _routeFn(elem, parent);
  }
}

Route.createRouteFromReactElement = createRouteElement(Route);
IndexRoute.createRouteFromReactElement = createRouteElement(IndexRoute);
module.exports.Link = Link;
module.exports.Route = Route;
module.exports.IndexRoute = IndexRoute;
module.exports.Redirect = Redirect;
module.exports.IndexRedirect = IndexRedirect;

let routeId = 0;
const HISTORY = {},
  HISTORY_ROUTES = {};

/* Export the getQuery() function */
export function getQuery() {
  let q = window.location.href.split('?')[1] || '';
  try {
    return qs.parse(q);
  } catch (e) {
    return {};
  }
}

export function clearPersistentPayload(key) {
  if (typeof key !== 'string') {
    persistentPayload = null;
  } else if (persistentPayload) {
    delete persistentPayload[key];
    if (Object.keys(persistentPayload).length === 0) {
      persistentPayload = null;
    }
  }
}

export function setPersistentPayload(v, shouldReload, _r) {
  if (typeof v === 'object' && v) {
    persistentPayload = Object.assign({}, persistentPayload, v);
  } else {
    persistentPayload = null
  }
  if (shouldReload !== false) {
    reload();
  }
}

/* Creates a new browser history */
export function getHistory(name) {
  if (typeof name === 'undefined') return browserHistory;
  if (!HISTORY[name]) {
    HISTORY[name] = useRouterHistory(createMemoryHistory)();
    HISTORY[name].listen((location) => {
      HISTORY[name].location = location.pathname;
    });
  }
  return HISTORY[name];
}


/* Mounts a custom history */
export function mountHistory(name) {
  let obj = HISTORY[name];
  if (!obj) return;
  let routes = HISTORY_ROUTES[name] || [];
  return (
    <Router history={obj}>
      {routes}
    </Router>
  )
}


/* Register a new route and pushes it in the stack
* Options:
*   - opt.root=true => place the route in the root routes. Only
*           works for non-named routers
* */
export function add(routeObj, _name, opt) {
  if (typeof routeObj !== 'object' || !routeObj) {
    console.warn(`router usage: add(Route)`);
    return false;
  }
  if (typeof _name === 'object' && _name) {
    opt = _name;
  }
  if (typeof opt !== 'object' || !opt) opt = {};
  let newObj = React.cloneElement(routeObj, {
    key: routeId++
  });
  if (typeof _name === 'string') {
    getHistory(_name);
    if (!HISTORY_ROUTES[_name]) HISTORY_ROUTES[_name] = [];
    HISTORY_ROUTES[_name].push(newObj);
  } else {
    if (opt && opt.root === true) {
      ROOT_ROUTES.push(newObj);
    } else {
      ROUTES.push(newObj);
    }
  }
}

/* Combine an array of routes */
export function combine(routes) {
  if (!(routes instanceof Array)) routes = Array.prototype.slice.call(arguments);
  return routes.map((item) => {
    return React.cloneElement(item, {
      key: routeId++
    })
  });
}

export function hasBack() {
  return hasRouterBack;
}

export function back(name) {
  let _history = (typeof name === 'string' && name.indexOf('/') === -1 ? getHistory(name) : browserHistory);
  if (hasRouterBack || _history.length > 2) {
    return _history.goBack();
  }
  if (name === true) {
    let cnt;
    if (getCurrentPath().split('/').pop() === 'edit') {
      cnt = 2;
    }
    let nextPath = getPreviousPath(cnt);
    return navigate(nextPath, null);
  }
  if (typeof name === 'string') {
    return navigate(name, null);
  }
  return navigate('/', null, name);
}

let hasRouterBack = false;

/* Manually reload the current path */
export function reload(_payload) {
  return navigate(getCurrentPath(), _payload);
}

/* Manually navigate to a path */
export function navigate(path, payload, _name) {
  if (typeof path !== 'string' || !path) {
    console.warn(`router usage: navigate(path, payload)`);
    return this;
  }
  hasRouterBack = true;
  if (typeof persistentPayload === 'object' && persistentPayload) {
    payload = Object.assign({}, persistentPayload, payload || {});
  }
  if (typeof payload === 'object' && payload) {
    let query = qs.stringify(payload);
    (path.indexOf('?') === -1 ? path += '?' : path += '&');
    path += query;
  }
  if (typeof _name === 'string') {
    let _history = getHistory(_name);
    _history.push(path, payload || {});
  } else {
    browserHistory.push(path, payload || {});
    tredux.dispatch(tredux.actions['core.ui'].changePath(path));
  }
  return this;
}

/*
 * Goes back in the URL path with 1. (performs a pop() on last /item.
 * */
export function getPreviousPath(cnt) {
  let current = getCurrentPath(0);
  let tmp = current.split('/');
  if (tmp.length === 1) return current;
  if (typeof cnt === 'number' && cnt > 1) {
    for (let i = 0; i < cnt; i++) {
      tmp.pop();
    }
  } else {
    tmp.pop();
  }
  return tmp.join('/');
}

/* Returns the computed router */
export function mount(_name) {
  if (typeof _name !== 'string') {
    let children = [];
    if (ROOT_ROUTES.length > 0) {
      for (let i = 0, len = ROOT_ROUTES.length; i < len; i++) {
        children.push(ROOT_ROUTES[i]);
      }
    }
    children.push(
      <Route path="/" key="1">
        {ROUTES}
      </Route>
    );
    children.push(<Route path="*" component={PageNotFound} key="2"/>);
    return (
      <Router history={browserHistory}>
        {children}
      </Router>
    )
  }
  let history = getHistory(_name),
    routes = HISTORY_ROUTES[_name] || [];
  return (
    <Router history={history}>
      {routes}
    </Router>
  );
}

/**
 * Appends a path to the current path
 * */
export function appendPath(sub) {
  let current = getCurrentPath();
  if (current.charAt(current.length - 1) === '/') current = current.substr(0, current.length - 1);
  if (sub.charAt(0) !== '/') sub = '/' + sub;
  // check if we have /app/dashboard with sub /dashboard, to avoid /app/dashboard/dashboard
  let tmp1 = current.split('/'),
    tmp2 = sub.split('/');
  if (tmp1.pop() === tmp2.pop()) return current;
  return current + sub;
}

/**
 * Returns the current base path
 * */
export function getBasePath() {
  let href = window.location.pathname.split('/');
  return href[1] || '';
}

/**
 * Returns the current path
 * */
export function getCurrentPath() {
  let href = window.location.pathname.split('?')[0];
  if (href.charAt(0) !== '/') href = '/' + href;
  return href;
}

/*
 * Checks if the given path is part of the current path
 * */
export function containsPath(path) {
  let current = getCurrentPath();
  return (current.indexOf(path) === 0);
}


/*
 * Wrapper function for the router onEnter/onLeave calling.
 * */
export function enter(pageClass, enterFn, isLoad) {
  if (!enterFn) {
    try {
      if (typeof pageClass.enter === 'function') {
        enterFn = pageClass.enter;
      }
    } catch (e) {
      if (typeof pageClass === 'object' && pageClass && typeof pageClass.enter === 'function') {
        enterFn = pageClass.enter;
      } else if (typeof pageClass === 'function') {
        enterFn = pageClass;
      }
    }
  }
  let active = false;
  return function onEnter(state, r, fn) {
    if (!enterFn) {
      return fn && fn();
    }
    if (active) {
      return fn && fn();
    }
    if (isLoad) {
      let current = getCurrentPath(),
        tmp = current.split('/'),
        final = tmp.pop(),
        prevFinal = tmp.pop();
      if (final === 'add' || final === 'edit' || prevFinal === 'add') {
        return fn && fn();
      }
    }
    active = true;
    setTimeout(() => {
      active = false;
    }, 100);
    util.loading();
    try {
      let fnPromise = enterFn(state, (e) => {
        try {
          if (e) appError(e);
          fn && fn(e);
          util.loading(false);
        } catch (e) {
          util.loading(false);
        }
      });
      if (typeof fnPromise === 'object' && fnPromise && typeof fnPromise.then === 'function' && typeof fnPromise.catch === 'function') {
        fnPromise.then(() => {
          fn && fn();
          util.loading(false);
        }, (e) => {
          try {
            appError(e);
            fn && fn(e);
            util.loading(false);
          } catch (e) {
            util.loading(false);
          }
        }).catch((e) => {
          appError(e);
          fn && fn(e);
          util.loading(false);
        });
        return false;
      }
      util.loading(false);
      fn && fn();
      return false;
    } catch (e) {
      appError(e);
      fn && fn(e);
      util.loading(false);
    }
    return false;
  }
}

/* Triggered ONLY when state.location.action = POP */
export function change(pageClass) {
  let enterFn = false;
  try {
    if (typeof pageClass.enter === 'function') {
      enterFn = pageClass.enter;
    }
  } catch (e) {
    if (typeof pageClass === 'object' && pageClass && typeof pageClass.enter === 'function') {
      enterFn = pageClass.enter;
    } else if (typeof pageClass === 'function') {
      enterFn = pageClass;
    }
  }
  let active = false;
  return function (state, next, r, fn) {
    if (!enterFn) {
      return fn && fn();
    }
    if (active) return fn && fn();
    active = true;
    setTimeout(() => {
      active = false;
    }, 100);
    let isOk = false;
    // we only treat BACK requests.
    let currentPath = getCurrentPath(),
      statePath = state.location.pathname;
    if (currentPath.length <= statePath.length) {
      isOk = true;
    }
    if (!isOk) return fn && fn();
    util.loading();
    try {
      let fnPromise = enterFn(state, fn);
      if (typeof fnPromise === 'object' && fnPromise && typeof fnPromise.then === 'function' && typeof fnPromise.catch === 'function') {
        fnPromise.then(() => {
          fn && fn();
          util.loading(false);
        }, (e) => {
          try {
            appError(e);
            fn && fn(e);
          } catch (e) {
          }
          util.loading(false);
        }).catch((e) => {
          appError(e);
          fn && fn(e);
          util.loading(false);
        });
      }
    } catch (e) {
      appError(e);
      fn && fn(e);
      util.loading(false);
    }
  }
}

export function leave(pageClass) {
  let enterFn = false;
  try {
    if (typeof pageClass.leave === 'function') {
      enterFn = pageClass.leave;
    }
  } catch (e) {
    if (typeof pageClass === 'object' && pageClass && typeof pageClass.leave === 'function') {
      enterFn = pageClass.leave;
    } else if (typeof pageClass === 'function') {
      enterFn = pageClass;
    }
  }
  let active = false;
  return function (state) {
    if (!enterFn) return;
    if (active === true) return;
    active = true;
    setTimeout(() => {
      try {
        enterFn(state);
      } catch (e) {
        appError(e);
      }
    }, 10);
    setTimeout(() => {
      active = false;
    }, 50);
  }
}

export function appError(e) {
  util.error(e.message);
  if (NODE_ENV !== 'development') {
    back();
  }
  tredux.emit('ERROR', {
    error: e
  });
}

tredux.addListener('core.ui.isBooted', () => {
  browserHistory.listen((data) => {
    if (data.action !== 'POP') return;
    let uiAct = tredux.actions['core.ui'];
    if (!uiAct) return;
    let state = tredux.getState()['core.ui'];
    if (state.url === data.pathname) return;
    tredux.dispatch(uiAct.changePath(data.pathname));
  });
});

