'use strict';
/**
 * General utilities
 */
window.noop = function () {
};

const router = require('router'),
  moment = require('moment'),
  async = require('async'),
  config = require('config'),
  countries = require('tredux-material/countries'),
  querystring = require('querystring'),
  tredux = require('tredux');

/**
 * Attach the standard handleFind function to tredux.
 * This will always use payload.result[] and payload.meta{}
 * to set in the current state.
 * state.meta = payload.meta || {}
 * state.items = payload.result || []
 * */
tredux.handleFind = (state, payload) => {
  if (typeof payload !== 'object' || !payload) payload = {};
  if (typeof payload.type === 'string' && payload.hasOwnProperty('payload')) {
    payload = payload.payload || {};
  }
  if (payload instanceof Array) {
    if (typeof state.meta !== 'undefined') {
      state.meta = {};
    }
    state.items = payload;
  } else if (payload.result instanceof Array) {
    state.items = payload.result || [];
    if (typeof state.meta !== 'undefined') {
      state.meta = payload.meta || {};
    }
  } else if (payload instanceof Array) {
    state.items = payload;
    if (typeof state.meta !== 'undefined') {
      state.meta = {};
    }
  }
};

/**
 * Attach the standard handleRead function to tredux.
 * This will always use payload.result{} to place the given object in item.
 * If object is not present, or is empty, set item to null
 * */
tredux.handleRead = (state, payload) => {
  if (typeof payload !== 'object' || !payload) payload = {};
  if (typeof payload.type === 'string' && payload.hasOwnProperty('payload')) {
    payload = payload.payload || {};
  }
  if (typeof payload.result === 'object' && payload.result && Object.keys(payload) === 1) {
    payload = payload.result;
  }
  if (Object.keys(payload).length === 0) {
    state.item = null;
  } else {
    state.item = payload;
  }
}

/**
 * Attaches the standard handleFilter function to tredux. This will
 * set the state.filter from payload{}. If payload is empty,
 * reset filter.
 * */
tredux.handleFilter = (state, payload) => {
  if (typeof payload !== 'object' || !payload) payload = {};
  state.filter = payload || {};
};

/**
 * Attaches the standard handleForm function to a tredux reducer.
 * This will work with payload{} and place it under state.form.
 * If payload is empty, we reset form.
 * */
tredux.handleForm = (state, payload) => {
  if (typeof payload !== 'object' || !payload) payload = {};
  state.form = payload || {};
};


/* Any a href that only has the path will go with the router */
$(document).on('click', 'a[href]', function (e) {
  let href = $(this).attr('href');
  if (href.charAt(0) !== '/' || $(this).attr('target')) return;
  e.preventDefault();
  e.stopPropagation();
  if ($(this).attr('data-redirect')) {
    window.location.href = href;
  } else {
    router.navigate(href);
  }
  return false;
});

export function getCountry(code) {
  return countries.MAP[code] || '';
}

/*
 CONFIRMATIONS
 OPTIONS
 - title: the title to use, default to Confirmation
 - message - the message to display
 - button - the delete button text
 - cancel - the cancel text
 - render - the render function to use in stead of message.
 - content - the actual react content
 - close - the close callback function
 */
export function confirm(opt, fn) {
  if (typeof opt === 'function') fn = opt;
  if (typeof opt !== 'object' || !opt) opt = {};
  if (typeof opt.fn !== 'function') opt.fn = noop;
  opt.fn = fn;
  tredux.emit('CONFIRMATION', opt);
}



export function getPayload() {
  let qs = window.location.href.split('?')[1] || '';
  qs = qs.split('#')[0];
  let payload = querystring.parse(qs);
  if (typeof payload.params === 'string') {  // atob it
    try {
      let params = atob(payload.params);
      delete payload.params;
      params = JSON.parse(params);
      Object.keys(params).forEach((name) => payload[name] = params[name]);
    } catch (e) {
    }
  }
  return payload;
}

/**
 * Show/hide the loading screen blocking the user's view.
 * */
let isLoading = false,
  isFetching = false;

export function loading(visible) {
  if (visible !== false) {
    isLoading = true;
    tredux.emit(tredux.actions('core.ui').showLoading(true));
  } else {
    isLoading = false;
    tredux.emit(tredux.actions('core.ui').hideLoading());
  }
}

module.exports.isLoading = function () {
  return isLoading;
};

export function fetching(visible) {
  if (visible !== false) {
    isFetching = false;
    tredux.emit(tredux.actions('core.ui').showLoading(false));
  } else {
    isFetching = true;
    tredux.emit(tredux.actions('core.ui').hideLoading());
  }
}

module.exports.isFetching = function() {
  return isFetching;
};

/* TOASTS */
/* PREVENT TOAST */
let isToastBlocked = false;

export function blockToast(v) {
  isToastBlocked = (typeof v === 'boolean' ? v : true);
}

export function error(msg) {
  if (isToastBlocked) return;
  if (typeof msg === 'object' && msg && msg.message) {
    msg = msg.message;
  }
  tredux.emit('TOAST', {
    type: 'error',
    message: msg
  });
}

export function warning(msg) {
  if (isToastBlocked) return;
  tredux.emit('TOAST', {
    type: 'warning',
    message: msg
  });
}

export function info(msg) {
  if (isToastBlocked) return;
  tredux.emit('TOAST', {
    type: 'success',
    message: msg
  });
}


export function capitalize(str) {
  if (typeof str === 'undefined' || str === null) return '';
  str = str.toString();
  if (str.length === 0) return str;
  str = str.toLowerCase();
  return str.charAt(0).toUpperCase() + str.substr(1);
}

export function shortFormat(dt, _default) {
  if (typeof dt === 'undefined' || dt == null) return _default || '-';
  let date = new Date(dt);
  if (isNaN(date.getTime())) return _default || '-';
  let res = moment(date).format("MMM Do, YYYY HH:mm");
  return res;
}

/* Fetches the dateformat. */
export function dateFormat(dt, _default) {
  if (typeof dt === 'undefined' || dt == null) return _default || '-';
  let date = new Date(dt);
  if (isNaN(date.getTime())) return _default || '-';
  let res = moment(date).format('MMM Do, YYYY');
  return res;
}

/* Fetches the time format */
export function timeFormat(dt, _default) {
  if (typeof dt === 'undefined' || dt == null) return _default || '-';
  let date = new Date(dt);
  if (isNaN(date.getTime())) return _default || '-';
  let res = moment(date).format('HH:mm');
  return res;
}

export function fullFormat(dt, _default) {
  let date = dateFormat(dt),
    time = timeFormat(dt);
  if (!date || date === '-') return _default || '-';
  if (!time || time === '-') return _default || '-';
  let res = date + ' at ' + time;
  return res || _default;
}

export function numberFormat(n, _currency) {
  let postfix = '';
  if (typeof n === 'string') {
    let tmp = n.split('.');
    if (tmp.length === 2) {
      postfix = tmp.pop();
    }
  }
  if (postfix) postfix = '.' + postfix;
  if (typeof n === 'string') n = parseInt(n, 10);
  if (typeof n !== 'number' || isNaN(n)) return '-';
  n = n.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
  if (typeof _currency === 'undefined') return n + postfix;
  if (_currency.toString().toLowerCase() === 'usd') _currency = '$';
  return _currency + n + postfix;
}

export function sizeFormat(bytes) {
  if (typeof bytes === 'undefined' || bytes === 0) return '-';
  if (bytes == 0) return '0 Byte';
  if (typeof bytes !== 'number') bytes = parseInt(bytes);
  if (isNaN(bytes)) return '-';
  let k = 1000; // or 1024 for binary
  let dm = 2;
  let sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
  let i = Math.floor(Math.log(bytes) / Math.log(k));
  return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
}

/*
 * Returns the current UTC date, sa the one below.
 * */
export function dateStart(date) {
  if (typeof date !== 'object' || !date) {
    date = moment(date).toDate();
  }
  let y = date.getFullYear(),
    m = date.getMonth() + 1,
    d = date.getDate();
  let utc = `${y}-${m < 10 ? '0' : ''}${m}-${d < 10 ? '0' : ''}${d}`;
  utc += 'T00:00:00.000Z';
  return utc;
}

/*
 * Adds 23:59:59 to the day, so that it is end of day.
 * */
export function dateEnd(date, _asObject) {
  if (typeof date === 'object' && date) {
    date = date.toUTCString();
  }
  let dateObj = moment(date).utc().toDate();
  dateObj.setHours(23, 59, 59, 999);
  if (_asObject === true) return dateObj;
  let d = dateObj.toISOString();
  d = d.split('T')[0] + 'T23:59:59.999Z';
  return d;
}


/*
 * Promise chaining utility
 * */
export function series(items, _onComplete, _stopOnError) {
  if (!(items instanceof Array)) throw new Error('thorin.series: requires an array as the first argument.');
  let onComplete = (typeof _onComplete === 'function' ? _onComplete : false),
    stopOnError = (_onComplete === false ? false : (_stopOnError !== false));
  if (onComplete) {
    return doSeries(items, stopOnError, onComplete);
  }
  return new Promise((resolve, reject) => {
    doSeries(items, stopOnError, (e) => {
      if (e) return reject(e);
      resolve();
    });
  });
}

function doSeries(items, stopOnError, finalFn) {
  if (items.length === 0) return finalFn();
  let calls = [],
    isStopped = false,
    currentNext,
    stopError;

  function stopSeries(e) {
    if (typeof e !== 'undefined') {
      stopError = e;
    }
    isStopped = true;
    if (currentNext) {
      currentNext();
    }
    return null;
  }

  items.forEach((fn) => {
    if (typeof fn !== 'function') return;
    calls.push((done) => {
      if (isStopped) return done();
      let promiseObj;
      currentNext = done;
      try {
        promiseObj = fn(stopSeries);
      } catch (e) {
        if (stopOnError) {
          return done(e);
        }
        return done();
      }
      let isDone = false;
      if (typeof promiseObj === 'object' && promiseObj && typeof promiseObj.then === 'function' && typeof promiseObj.catch === 'function') {
        promiseObj.catch((e) => {
          if (isDone || isStopped) return null;
          isDone = true;
          if (stopOnError) {
            done(e);
            return null;
          }
          done();
          return null;
        });
        promiseObj.then((res) => {
          if (isDone || isStopped) return null;
          isDone = true;
          done(null, res);
          return null;
        }, (e) => {
          if (isDone || isStopped) return;
          isDone = true;
          if (stopOnError) {
            done(e);
            return null;
          }
          done();
          return null;
        });
        return null;
      } else {
        if (isDone || isStopped) return;
        isDone = true;
        done();
        return null;
      }
    });
  });
  async.series(calls, (e) => {
    if (isStopped) {
      if (stopError) {
        finalFn(stopError);
        return null;
      }
      finalFn();
      return null;
    }
    if (e) {
      finalFn(e);
      return null;
    }
    finalFn();
    return null;
  });
}

export function lighten(color, percent) {
  if (typeof color !== 'string') throw new Error("Invalid color for ui.lighten() [" + percent + ']');
  if (color.charAt(0) !== '#') color = '#' + color;
  let num = parseInt(color.slice(1), 16), amt = Math.round(2.55 * percent), R = (num >> 16) + amt,
    G = (num >> 8 & 0x00FF) + amt, B = (num & 0x0000FF) + amt;
  return "#" + (0x1000000 + (R < 255 ? R < 1 ? 0 : R : 255) * 0x10000 + (G < 255 ? G < 1 ? 0 : G : 255) * 0x100 + (B < 255 ? B < 1 ? 0 : B : 255)).toString(16).slice(1);
}

export function darken(color, percent) {
  try {
    return lighten(color, 0 - percent);
  } catch (e) {
    throw new Error("Invalid color for ui.darken() [" + percent + ']');
  }
}

let _t;

export function triggerResize() {
  clearTimeout(_t);
  _t = setTimeout(() => {
    try {
      let resizeEvent = window.document.createEvent('UIEvents');
      resizeEvent.initUIEvent('resize', true, false, window, 0);
      window.dispatchEvent(resizeEvent);
    } catch (e) {
      try {
        window.dispatchEvent(new Event('resize'));
      } catch (e) {
      }
    }
  }, 100);
}

export function camelCase(str) {
  if (typeof str !== 'string' || !str) return '';
  let tmp = str.split(/-|_/g);
  for (let i = 0; i < tmp.length; i++) {
    if (i === 0) {
      tmp[i] = tmp[i].toLowerCase();
      continue;
    }
    tmp[i] = tmp[i].charAt(0).toUpperCase() + tmp[i].substr(1).toLowerCase();
  }
  tmp = tmp.join('');
  return tmp;
}
