'use strict';
const React = require('react'),
  config = require('config'),
  fetcher = require('thorin-fetch'),
  api = require('api'),
  util = require('lib/util');

import FlatButton from 'material-ui/FlatButton';
import Dropzone from 'react-dropzone';

export default class AppFileUpload extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      dragHover: false,
      file: null,
      base64: ''
    }
  }

  checkMime(fileObj) {
    if (typeof this.props.mimeType === 'undefined') return true;
    let mimes = (this.props.mimeType instanceof Array ? this.props.mimeType : [this.props.mimeType]),
      type = fileObj.type;
    for (let i = 0; i < mimes.length; i++) {
      if (mimes[i] === type) return true;
    }
    return false;
  }

  getFile() {
    return this.state.file || null;
  }

  hasFile() {
    return (this.state.file != null);
  }

  handleRemove() {
    this.props.onDelete && this.props.onDelete();
  }

  cancel() {
    this.setState({
      file: null,
      base64: ''
    }, () => {
      if (this.root) {
        $(this.root).find("input[type='file']").trigger("change");
      }
      this.props.onChange && this.props.onChange(null);
      this.props.onPreview && this.props.onPreview(null);
    });
  }

  onUploaded(url) {
    this.setState({
      file: null
    }, () => {
      this.props.onUpload && this.props.onUpload(url);
    });
  }

  upload(actionName, payload) {
    if (typeof actionName !== 'string' || !actionName) {
      return Promise.reject(new Error('Action name is required on upload()'));
    }
    if (!this.hasFile()) {
      return Promise.resolve();
    }
    let uploadUrl,
      apiFn = api.fetch;
    return apiFn(actionName, {
      mime: this.state.file.type,
      ...payload
    }).then((res) => {
      if (!res.url) return Promise.reject(new Error('An error occurred while signing the upload request'));
      uploadUrl = res.url;
    }).then(() => {
      return fetcher.upload({
        credentials: false,
        url: uploadUrl
      }).send(this.state.file)
        .then((res) => {
          this.onUploaded(res.url);
          return res.url;
        });
    })
  }

  handleFile(files) {
    let fileObj = (files instanceof Array ? files[0] : files);
    if (!fileObj) return;
    if (!this.checkMime(fileObj)) {
      return this.props.onError && this.props.onError('The selected file is not supported');
    }
    this.setState({
      file: fileObj,
      dragHover: false
    }, () => {
      this.props.onChange && this.props.onChange(fileObj);
    });
    let reader = new FileReader();
    let base64data;
    if (fileObj.type && fileObj.type.indexOf('image/') !== -1) {
      reader.readAsDataURL(fileObj);
      reader.onloadend = () => {
        base64data = reader.result;
        this.setState({
          base64: base64data
        }, () => {
          this.props.onPreview && this.props.onPreview(base64data);
        });
      };
      reader.onerror = () => {
        this.props.onError && this.props.onError('An error occurred while opening your file');
      }
    }
  }

  handleSelect(e) {
    if (this.state.file) {
      return this.cancel();
    }
    this.dropzone.onClick(e);
  }

  getImage() {
    if (this.state.image) return this.state.image;
    if (this.props.image) return this.props.image;
    return null;
  }

  renderPreview(width, height) {
    let preview;
    let fileObj = this.state.file,
      fileText;
    if (this.state.base64) {
      preview = {
        backgroundColor: 'transparent',
        backgroundImage: `url("${this.state.base64}")`,
        display: 'block',
        width: width,
        height: height
      };
    } else {
      let image = this.getImage();
      if (image) {
        preview = {
          backgroundColor: 'transparent',
          backgroundImage: `url("${image}")`,
          display: 'block',
          width: width,
          height: height
        };
      } else if (fileObj) {
        preview = {
          display: 'block',
          width: width,
          height: height
        }
      }
    }
    if (fileObj && fileObj.type && fileObj.type.indexOf('image/') === -1) {
      fileText = fileObj.name;
    }
    return (
      <div className="preview-container" style={preview}>
        {fileText && <p>{fileText}</p>}
      </div>
    )
  }

  renderIcon(width, height) {
    if (this.state.file || this.getImage()) return;
    let widthText = this.props.widthText,
      heightText = this.props.heightText,
      hasTitle = !!this.props.title;
    let iconStyle = Object.assign({}, {
      display: 'inline-block'
    }, this.props.iconStyle || {});
    return (
      <div className={classNames("icon-container", {
        'item-container-no-title': !hasTitle
      })} style={{width: width, height: height}}>
        <div className="icon-container-text">
          <i className={classNames("app-fileupload-icon", this.props.iconClassName)}/>
          <div style={iconStyle}>
            {this.props.title ? (
              <p>{this.props.title}</p>
            ) : ""}
            {widthText && heightText && (
              <p>{widthText} x {heightText} px</p>
            )}
          </div>
        </div>
      </div>
    )
  }

  renderDimensions(width, height) {
    if (!this.state.file && !this.getImage() && !this.state.base64) return;
    let widthText = this.props.widthText,
      heightText = this.props.heightText;
    if (!this.props.title && !widthText && !heightText) return;
    return (
      <div className="app-fileupload-size">
        <p>{this.props.title}</p>
        {widthText && heightText && (
          <p>{widthText} x {heightText} px</p>
        )}
      </div>
    )
  }

  render() {
    let width = parseInt(this.props.width) + 2,
      height = parseInt(this.props.height) + 2;
    let aspectRatio = Math.floor((width / height) * 100) / 100;
    let label = this.state.file ? 'cancel' : (this.getImage() ? 'change' : 'select');
    return (
      <div
        ref={(obj) => this.root = obj}
        className={classNames("app-fileupload", {
          "portrait-view": aspectRatio <= 1,
          'is-hover': this.state.dragHover
        }, this.props.className)}
        style={this.props.style}
      >
        <div className="app-fileupload-container">
          <Dropzone
            disabled={this.props.disabled}
            name={this.props.name}
            multiple={false}
            ref={(c) => this.dropzone = c}
            onDrop={this.handleFile.bind(this)}
            onDragEnter={() => this.setState({dragHover: true})}
            onDragLeave={() => this.setState({dragHover: false})}
            className="dropzone-area"
            style={{width: width, height: height}}
          >
            {this.renderIcon(width, height)}
            {this.renderPreview(width, height)}
            {this.renderDimensions(width, height)}
          </Dropzone>
          <div className="upload-actions">
            <FlatButton disabled={this.props.disabled} label={label} onClick={(e) => this.handleSelect(e)}/>
            {this.props.onDelete && !this.state.file && this.getImage() && (
              <FlatButton disabled={this.props.disabled} className="button-upload-remove" label="REMOVE"
                          onClick={(e) => this.handleRemove()}/>
            )}
          </div>
        </div>
      </div>
    )
  }
}
AppFileUpload.defaultProps = {
  iconClassName: 'ua-cloud-computing'
};

AppFileUpload.PropTypes = {
  widthText: React.PropTypes.string,
  heightText: React.PropTypes.string,
  width: React.PropTypes.number.isRequired,
  height: React.PropTypes.number.isRequired,
  mimeType: React.PropTypes.any,  // string or array(string)
  iconClassName: React.PropTypes.string,
  onChange: React.PropTypes.func,
  onDelete: React.PropTypes.func,
  onUpload: React.PropTypes.func,
  disabled: React.PropTypes.func,
  onPreview: React.PropTypes.func,
};

