'use strict';

const React = require('react'),
  router = require('router'),
  tredux = require('tredux');

import Paper from 'material-ui/Paper';
import Divider from 'material-ui/Divider';
import Menu from 'material-ui/Menu';
import MenuItem from 'material-ui/MenuItem';

class AppMenuItem extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isHovered: false,
      showSubMenu: false
    };
    //Bindings
    this.handleHover = this.handleHover.bind(this);
    this.handleClick = this.handleClick.bind(this);
    this.handleHoverLeave = this.handleHoverLeave.bind(this);
    this._destroyed = false;
  }

  componentWillUnmount() {
    this._destroyed = true;
  }

  componentDidMount() {
    tredux.addListener(tredux.actions['core.ui'].TYPE.SHOW_MENU, (e) => {
      if (this._destroyed) return;
      this.setState({
        showSubMenu: false
      });
    });
  }

  handleHover() {
    if (this._destroyed) return;
    this.setState({
      isHovered: true,
      showSubMenu: true
    });
  }

  handleHoverLeave() {
    if (this._destroyed) return;
    this.setState({
      isHovered: false,
      showSubMenu: false
    });
  }

  handleClick() {
    if (this._destroyed) return;
    this.props.onClick && this.props.onClick();
    if (this.props.href) {
      router.navigate(this.props.href);
    } else if (this.props.path) {
      let _p = this.props.path;
      if (_p.charAt(0) !== '/') _p = '/' + _p;
      router.navigate(_p);
    }
    if (!this.props.showSubMenu && !this.props.ui.menuOpened) {
      this.setState({
        showSubMenu: true
      });
    }
    if (!this.hasSubMenu() || this.isActive()) {
      this.handleHoverLeave();
    }
  }

  isActive() {
    if (typeof this.props.active === 'boolean') return this.props.active;
    let path = this.props.path && this.props.path.toLowerCase() || '',
      base = router.getBasePath(),
      current = router.getCurrentPath();
    if (path.charAt(0) !== '/') path = '/' + path;
    if (base.charAt(0) !== '/') base = '/' + base;
    if (current.charAt(0) !== '/') current = '/' + current;
    let isActive = base === path || current === path;
    return (this.hasSubMenu() && this.state.isHovered) || isActive;
  }

  hasSubMenu() {
    return this.props.subMenus && (this.props.subMenus instanceof Array) && this.props.subMenus.length > 0;
  }

  renderSubMenu() {
    if (!this.isActive() || !this.state.showSubMenu || this.props.ui.menuOpened || !this.hasSubMenu()) return;
    return (
      <Paper className={"sub-menu"} rounded={false} zDepth={1}>
        {this.renderSubMenuItems()}
      </Paper>
    )
  }

  renderSubMenuItems() {
    let submenuItems = [];
    let idx = 1;
    if (this.props.subMenus && this.props.subMenus instanceof Array) {
      let subMenus = this.props.subMenus;
      subMenus.forEach((item) => {
        let itemView = <MenuItem
          key={idx}
          primaryText={<div className="text-overflow">{item.label}</div>}
          className="sub-btn"
          onClick={(e) => {
            e.preventDefault();
            e.stopPropagation();
            this.handleHoverLeave();
            this.props.onClick && this.props.onClick(item.id);
          }}
        />;
        submenuItems.push(itemView);
        idx++;
      });
    }
    return submenuItems;
  }

  render() {
    let path = this.props.path,
      base = router.getBasePath(),
      current = router.getCurrentPath().toLowerCase(),
      isActive = false;
    if (typeof path === 'string') {
      path = path.toLowerCase();
      if (path.charAt(0) !== '/') path = '/' + path;
      if (base.charAt(0) !== '/') base = '/' + base;
      if (current.charAt(0) !== '/') current = '/' + current;
      isActive = base === path || current === path;
      if (this.props.activeSub === true) {
        if (current.indexOf(path) === 0) {
          isActive = true;
        }
      }
    }
    if (typeof this.props.active === 'boolean') {
      isActive = this.props.active;
    }
    let btnClass = classNames(
      'btn', {
        'btn-over': this.state.isHovered,
        'btn-active': isActive,
        'btn-no-icon': !this.props.icon
      }
    );
    return (
      <div
        className={btnClass}
        onMouseEnter={this.handleHover}
        onMouseLeave={this.handleHoverLeave}
        onClick={this.handleClick}
      >
        {this.renderIcon()}
        {this.renderTitle()}
        {this.renderSubMenu()}
      </div>
    )
  }

  renderIcon() {
    if (!this.props.icon) return;
    return (
      <div className="menu-icon">{this.props.icon}</div>
    )
  }

  renderTitle() {
    if (!this.submenu && !this.props.ui.menuOpened && !this.props.ui.menuMinimized) return;
    return (
      <div className="menu-title" title={this.props.name}>
        {this.props.name}
      </div>
    )
  }
}


module.exports = tredux.connect((state) => {
  return {
    ui: state['core.ui']
  }
}, AppMenuItem);

AppMenuItem.PropTypes = {
  icon: React.PropTypes.string,
  subMenus: React.PropTypes.array,
  active: React.PropTypes.bool,
  activeSub: React.PropTypes.bool,
  name: React.PropTypes.string,
  path: React.PropTypes.string,
  onClick: React.PropTypes.func
};
module.exports.Component = AppMenuItem;
