'use strict';
const React = require('react'),
  Row = require('./FiltrumRow');

import ClearIcon from 'material-ui/svg-icons/content/clear';
import TextField from 'core/components/TextField';
import Button from 'core/components/Button';


class BaseFilter extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      filter: {},
      activeFilter: {}, // the filter sent to the parent with onFilter()
      hasSearchText: false,
      isSearchFocused: false,
      isFilterRendered: false,
      isFilterFocused: false,
      isFilterVisible: false
    };
    if (props.filterOpened) {
      this.state.isFilterVisible = true;
      this.state.isFilterFocused = true;
    }
    this._fields = {};
    this._timers = {};
    this._blocks = {};
    this.hasFilter = this.hasFilter.bind(this);
    this.clear = this.clear.bind(this);
    this.filter = this.filter.bind(this);
    this.field = this.field.bind(this);
    this.set = this.set.bind(this);
    this.get = this.get.bind(this);
    this.change = this.change.bind(this);
    this.handleSearch = this.handleSearch.bind(this);
    this.handleFilterFocus = this.handleFilterFocus.bind(this);
    this.handleFilterVisible = this.handleFilterVisible.bind(this);
  }

  componentWillUnmount() {
    Object.keys(this._timers).forEach((t) => {
      clearTimeout(this._timers[t]);
    });
    Object.keys(this._blocks).forEach((t) => {
      clearTimeout(this._blocks[t]);
    });
    this._timers = {};
    this._fields = {};
    this._blocks = {};
  }

  /*
  * Checks if the toggle is active
  * */
  isToggleActive() {
    let filter = this.state.activeFilter;
    if (typeof filter !== 'object' || !filter) return false;
    let keys = Object.keys(filter);
    if (keys.length === 0) return false;
    let count = 0;
    for (let i = 0, len = keys.length; i < len; i++) {
      if (keys[i] === this.props.searchName) continue;
      if (keys[i] === 'page') continue;
      if (keys[i] === 'limit') continue;
      let kv = filter[keys[i]];
      if (kv instanceof Array && kv.length === 0) continue;
      if (this._primary && keys[i] === this._primary.props.name) continue;
      count++;
    }
    return (count > 0);
  }

  /*
  * Renders the Search textfield
  * */
  _renderSearchField() {
    return (
      <div className="app-filtrum-search">
        <TextField
          ref={(c) => this._search = c}
          defaultValue={this.props.searchValue}
          onKeyPress={(e) => {
            if (e.charCode === 13 && this._search) {
              this.handleSearch(this._search.getValue());
            }
          }}
          onChange={(val) => {
            if (typeof val !== 'string') return;
            let hasSearchText = (val !== '');
            let upd = {};
            if (this.state.hasSearchText !== hasSearchText) {
              upd.hasSearchText = hasSearchText;
            }
            if (!hasSearchText && typeof this.state.filter[this.props.searchName] === 'string') {
              let filter = this.state.filter || {};
              delete filter[this.props.searchName];
              upd.filter = filter;
            }
            if (hasSearchText) {
              upd.filter = this.state.filter || {};
              upd.filter[this.props.searchName] = val;
            }
            this.setState(upd);
          }}
          onFocus={() => this.handleSearchFocus(true)}
          onBlur={() => this.handleSearchFocus(false)}
          name={this.props.searchName}
          hintText={this.props.labelSearch}
          autoComplete="off"
        />
        {this.state.hasSearchText ? (
          <ClearIcon onClick={() => {
            this.handleSearch('');
            this._search && this._search.setValue('');
          }} className="app-filtrum-search-clear"/>
        ) : ''}
      </div>
    )
  }

  /*
  * Renders the filtrum content block
  * */
  _renderContentBlock(items) {
    if (!this.state.isFilterRendered) return;
    if (!items) items = this.getItems();
    if (items.length === 0) return;
    return (
      <div className={classNames('app-filtrum-content', {
        'app-filtrum-content-visible': this.state.isFilterVisible
      })}>
        <div className="app-filtrum-items">
          {items}
        </div>
        <div className="app-filtrum-actions">
          {this.props.canClear && (
            <Button
              label={this.props.labelClear}
              flat={true}
              onClick={this.clear}
            />
          )}
          <Button
            primary={true}
            label={this.props.labelFilter}
            style={{marginLeft: 8}}
            onClick={() => {
              if (this._search) {
                return this.handleSearch(this._search.getValue(), this.filter);
              }
              this.filter();
            }}
          />
        </div>
      </div>
    )
  }

  /*
  * Renders the filter overlay to close the filter block on click outside
  * */
  _renderOverlay() {
    if (!this.state.isFilterVisible) return;
    return (
      <div
        className="app-filtrum-overlay"
        onClick={() => {
          if (this._searchFocused) return;
          let upd = {
            isFilterVisible: false,
            isFilterFocused: false
          };
          if (this.type !== 'filter-only' && this.state.hasSearchText) {
            upd.isFilterFocused = true;
          }
          this.setState(upd);
        }}
      />
    )
  }

  /*
  * Returns the children that are of type FiltrumRow
  * */
  getItems() {
    let items = [];
    let children = (this.props.children instanceof Array ? this.props.children : [this.props.children]);
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (typeof child !== 'object' || !child) continue;
      if (child.type !== Row) continue;
      if (!child.props.children) continue;
      items.push(child);
    }
    return items;
  }

  /*
  * Simply checks if we have a filter in our state.
  * */
  hasFilter(ignoreKey) {
    if (typeof this.state.filter !== 'object' || !this.state.filter) return false;
    let keys = Object.keys(this.state.filter);
    if (keys.length === 0) return false;
    let _count = 0;
    for (let i = 0, len = keys.length; i < len; i++) {
      let key = keys[i],
        value = this.state.filter[key];
      if (typeof ignoreKey === 'string' && key === ignoreKey) continue;
      if (value === 'ALL' || value === '') continue;
      _count++;
    }
    return (_count !== 0);
  }

  /*
  * Simply checks if the filter has the "search" term.
  * */
  hasSearch() {
    if (typeof this.state.filter !== 'object' || !this.state.filter) return false;
    if (typeof this.state.filter[this.props.searchName] !== 'string') return false;
    if (this.state.filter[this.props.searchName].trim() === '') return false;
    return true;
  }

  /*
  * Blocks an event of happening for a short period of time
  * */
  _block(name, _ms) {
    if (this._blocks[name]) clearTimeout(this._blocks[name]);
    this._blocks[name] = setTimeout(() => {
      delete this._blocks[name];
    }, _ms || 50);
  }

  /*
  * Registers a setTimeout for the given function, to delay actions
  * */
  _delay(name, fn, _ms) {
    this._clear(name);
    if (this._blocks[name]) return;
    this._timers[name] = setTimeout(() => {
      this._clear(name);
      if (this._blocks[name]) return;
      fn();
    }, _ms || 20);
  }

  /*
  * Clears a previously registered delayed function
  * */
  _clear(name) {
    if (this._timers[name]) {
      clearTimeout(this._timers[name]);
      delete this._timers[name];
    }
  }

  /*
  * Returns all the class names of the filter.
  * */
  getClassNames(items) {
    let hasFilter = this.hasFilter();
    return classNames(this.props.className, 'app-filtrum', `app-filtrum-${this.type}`, {
      'app-filtrum-focused': this.props.searchExpand === false ? true : (this.state.isFilterFocused || this.state.isSearchFocused),
      'app-filtrum-search-focused': this.state.isSearchFocused,
      'app-filtrum-visible': this.state.isFilterVisible,
      'app-filtrum-has-items': items instanceof Array && items.length > 0,
      'app-filtrum-has-data': hasFilter,
      'app-filtrum-no-data': !hasFilter
    });
  }

  /*
  * Triggered when the search input is focused
  * */
  handleSearchFocus(isSearchFocused) {
    if (this._searchFocused) return;
    this._searchFocused = true;
    setTimeout(() => {
      delete this._searchFocused;
    }, 100);
    if (this.state.isSearchFocused === isSearchFocused) {
      return this.handleFilterFocus(isSearchFocused);
    }
    this.setState({
      isSearchFocused
    }, () => {
      this.handleFilterFocus(isSearchFocused);
    });
  }

  /*
  * Triggered when the filter box is focused
  * */
  handleFilterFocus(isFilterFocused) {
    this._delay('filterFocus', () => {
      //if (!isFilterFocused && this.hasFilter()) return;
      //if (!isFilterFocused && this.state.isFilterVisible) return;
      let upd = {
        isFilterFocused
      };
      /* Check if we have a search. If so and we have a value, prevent */
      if (!isFilterFocused && this._search) {
        upd.isFilterFocused = this.state.hasSearchText;
      }
      if (this.state.isFilterVisible) {
        upd.isFilterFocused = true;
      }
      if (!upd.isFilterFocused) {
        upd.isFilterVisible = false;
      }
      this.setState(upd);
    }, isFilterFocused ? 50 : undefined);
  }

  /*
  * Triggered on search text change
  * */
  handleSearch(search, fn) {
    let filter = Object.assign({}, this.state.filter);
    if (typeof search !== 'string' || !search || search.trim() === '') {
      if (this.state.hasSearchText) {
        this.setState({
          hasSearchText: false
        });
      }
      if (typeof filter[this.props.searchName] === 'undefined') return fn && fn() || this.filter();
      delete filter[this.props.searchName];
    } else {
      if (filter[this.props.searchName] === search) return fn && fn() || this.filter();
      filter[this.props.searchName] = search;
    }
    let upd = {
      filter
    };
    if (filter.search && !this.state.isFilterFocused) {
      upd.isFilterFocused = true;
    }
    if (!filter.search && !this.state.isFilterVisible && !this.hasFilter()) {
      upd.isFilterFocused = false;
    }
    this.setState(upd, fn || this.filter);
  }

  /*
  * Triggered by the FilterToggle and checks the filter visibility state
  * */
  handleFilterVisible(isFilterVisible) {
    this._block('filterFocus');
    this._delay('filterVisible', () => {
      let upd = {
        isFilterVisible
      };
      if (isFilterVisible && !this.state.isFilterFocused) {
        upd.isFilterFocused = true;
      }
      if (this.state.isFilterFocused && !isFilterVisible) {
        upd.isFilterFocused = false;
      }
      this.setState(upd);
    }, 10);
  }

  /*
  * Clears all the filter and resets its focused state.
  * */
  clear(includeSearch) {
    let filter = {};
    if (includeSearch !== true && this._search) {
      let _val = this._search.getValue();
      if (_val && _val.trim() !== '') {
        filter[this.props.searchName] = _val;
      } else {
        delete filter[this.props.searchName];
      }
    }
    /* Clear our fields */
    Object.keys(this._fields).forEach((fieldName) => {
      let fieldObj = this._fields[fieldName];
      if (typeof fieldObj.setValue === 'function') {
        let val = '';
        if (typeof fieldObj.props.defaultValue !== 'undefined' && fieldObj.props.defaultValue !== null && fieldObj.props.defaultValue !== '') {
          val = fieldObj.props.defaultValue;
        }
        // Check the defaultFieldValue
        let primaryFieldObj = this._fields['__primary_field'];
        if (primaryFieldObj && primaryFieldObj.props.name === fieldName) {
          val = primaryFieldObj.props.defaultValue || '';
        }
        if (val !== '' && val !== 'ALL' && fieldName !== '__primary_field') {
          filter[fieldName] = val;
        }
        fieldObj.setValue(val);
      } else if (typeof fieldObj.clear === 'function') {
        fieldObj.clear();
      } else {
        warn(`Field ${fieldObj.props.name} does not have a clear function`);
        warn(fieldObj);
      }
    });
    let upd = {
      filter,
      isFilterFocused: false,
      isFilterVisible: false
    };
    if (this._search) {
      if (includeSearch === true) {
        upd.hasSearchText = false;
      } else if (this.state.hasSearchText) {
        upd.isFilterFocused = true;
      } else {
        upd.isFilterFocused = false;
      }
    }
    this._delay('_filter', () => {
      this.setState(upd, () => {
        if (includeSearch === true && this._search) {
          this._search.setValue('');
        }
        this.filter();
      });
    }, 20);
  }

  /*
  * Manually trigger the this.props.onFilter() function
  * */
  filter() {
    this._delay('filter', () => {
      let clone = JSON.parse(JSON.stringify(this.state.filter || {})),
        upd = {};
      if (this.state.isFilterVisible) {
        upd.isFilterVisible = false;
      }
      if (this.state.hasSearchText) {
        upd.isFilterFocusd = true;
      } else {
        upd.isFilterFocused = false;
      }
      if (!this.state.hasSearchText && this.state.isFilterFocused && (this.type === 'filter-only' || Object.keys(clone).length === 0)) {
        upd.isFilterFocused = false;
      }
      upd.activeFilter = JSON.parse(JSON.stringify(this.state.filter));
      if (Object.keys(upd).length > 0) {
        this.setState(upd);
      }
      this.props.onFilter && this.props.onFilter(clone);
    });
  }

  /*
  * Compatibility type
  * */
  getPayload() {
    let clone = JSON.parse(JSON.stringify(this.state.filter || {}));
    return clone;
  }


  /*
  * Manually set a filter field
  * */
  set(key, value, fn) {
    if (typeof key !== 'string' || !key) return fn && fn();
    this.props.onChange && this.props.onChange(key, value);
    if (typeof value === 'undefined' || value === null || value === 'ALL' || (typeof value === 'string' && value.trim() === '')) {
      let filter = Object.assign({}, this.state.filter);
      delete filter[key];
      this.setState({
        filter: filter
      }, fn);
      return;
    }
    let upd = Object.assign({}, this.state.filter);
    upd[key] = value;
    this.setState({
      filter: upd
    }, fn);
  }

  /*
  * Manually retrieve a filter field
  * */
  get(key) {
    if (typeof key !== 'string' || !key) return null;
    if (typeof this.state.filter[key] === 'undefined') return null;
    return this.state.filter[key];
  }

  /*
  * Triggered by a field during its onChange
  * */
  change(key) {
    return (val) => {
      this.set(key, val);
    }
  }

  /*
  * Manually register a field with its reference to the filter
  * */
  field(obj, __name) {
    if (!obj) return;
    if (!obj.props || !obj.props.name) {
      warn(`field() received a field that has no name. Skipping`);
      return;
    }
    let updateValue = true;
    if (this._fields[__name || obj.props.name]) {
      updateValue = false;
    }
    this._fields[__name || obj.props.name] = obj;
    if (updateValue && typeof obj.getValue === 'function' && obj.getValue) {
      let val = obj.getValue();
      if (typeof val === 'undefined' || val === null || val === 'ALL') return;
      if (typeof val === 'string' && val.trim() === '') return;
      this.state.filter[obj.props.name] = val;  // Do not use setState(), since we do not want rerender
      this.state.activeFilter[obj.props.name] = val;
      this._delay('activeFilter', () => {
        this.setState({
          activeFilter: JSON.parse(JSON.stringify(this.state.activeFilter))
        });
      });
    }
  }

  _warn(msg, logOnce) {
    if (logOnce === true) {
      if (!this._warnings) this._warnings = {};
      if (this._warnings[msg]) return;
      this._warnings[msg] = true;
    }
    warn(msg);
  }
}

module.exports = BaseFilter;

function warn(msg) {
  console.warn && console.warn(`Tredux-material.Filter: ${msg}`);
}
