'use strict';
const React = require('react');

import {
  Stepper as MuiStepper,
  Step as MuiStep,
  StepLabel as MuiStepLabel
} from 'material-ui/Stepper';

import CircularProgress from 'material-ui/CircularProgress';
import WarningIcon from 'material-ui/svg-icons/alert/warning';

const Step = require('./Step'),
  Summary = require('./Summary'),
  Content = require('./Content'),
  Actions = require('./Actions');

class AppStepper extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      canNext: null,  // can be overridden
      isLoading: props.loading || false,
      activeStep: 0,
      completed: [], // array of step index that are completed
      errors: []      // array of step index that are errors
    };
    this._stepMapIdx = {}; // map of {stepIdx:stepId}
    this._stepMap = {};   // map of {stepId:stepIdx}
    this._steps = [];
    this._summaries = {}; // a map of {stepPosition:summaryComponent}
    this._nextSummaryActive = false;
    this._getSteps = this._getSteps.bind(this);
    this.next = this.next.bind(this);
    this.back = this.back.bind(this);
  }

  componentDidMount() {
    /* Verify if we have a summary on position 0 */
    if (this._summaries[0]) {
      this._summaries[0].show();
    }
    if (typeof this.props.step !== 'undefined') {
      this.setStep(this.props.step);
    }
  }

  componentWillReceiveProps(nextProps) {
    let upd = {};
    if (typeof nextProps.loading === 'boolean') {
      upd.loading = nextProps.loading;
    }
    if (Object.keys(upd).length === 0) return;
    this.setState(upd);
  }

  /**
   * Checks if a given step is completed
   * */
  isCompleted(pos) {
    return this.state.completed.indexOf(this._getStepIdx(pos)) !== -1;
  }

  /**
   * Manually override the canNext status
   * */
  enableNext() {
    if (typeof this.state.canNext !== 'boolean') return;
    let st = this.state;
    delete st.canNext;
    this.setState(st);
  }

  disableNext() {
    this.setState({
      canNext: false
    });
  }

  /**
   * Verifies if we can move onwards to the next step
   * */
  canNext() {
    if (this.props.canNext) {
      return this.props.canNext(this._getStepId(this.state.activeStep));
    }
    if (this.state.errors.indexOf(this.state.activeStep) !== -1) return false;
    if (this.state.isLoading) return false;
    return true;
  }

  /**
   * Verifies if we can save
   * */
  canSave() {
    if (this.props.canSave) {
      return this.props.canNext(this._getStepId(this.state.activeStep));
    }
    return true;
  }

  /**
   * Verifies if we can move back to the previous step.
   * */
  canBack() {
    if (this.props.canBack) {
      return this.props.canBack(this._getStepId(this.state.activeStep));
    }
    if (this.state.activeStep === 0) return false;
    if (this.state.isLoading) return false;
    return true;
  }

  /**
   * Verifies if a given step is in error state
   * */
  isError(step) {
    return this.state.errors.indexOf(this._getStepIdx(step)) !== -1;
  }

  /**
   * Sets the given step as the active step, bypassing any confirmations or canNext
   * */
  setStep(step) {
    let stepIdx = this._getStepIdx(step);
    if (typeof stepIdx !== 'number' || !stepIdx) return false;
    let completed = [];
    for (let i = 0, len = this._steps.length; i < len; i++) {
      if (i === stepIdx) break;
      completed.push(i);
    }
    let upd = {
      completed,
      activeStep: stepIdx
    };
    this.setState(upd, () => {
      this.props.onChange && this.props.onChange(this._getStepId(stepIdx));
    });
  }

  /**
   * Enables/disables loading state of the stepper
   * */
  loading(val) {
    if (typeof val !== 'boolean') val = true;
    this.setState({
      isLoading: val
    });
  }

  /**
   * Marks the given step as an error
   * */
  error(step, _isActive) {
    step = this._getStepIdx(step);
    if (typeof _isActive !== 'boolean') _isActive = true;
    let eIdx = this.state.errors.indexOf(step);
    if (_isActive && eIdx !== -1) return;
    let errors = this.state.errors.concat([]);
    if (_isActive) {
      if (eIdx !== -1) return;
      errors.push(step);
    } else {
      if (eIdx === -1) return;
      errors.splice(eIdx, 1);
    }
    this.setState({
      errors
    });
  }

  /**
   * Tries to go forward to the next step
   * */
  async next() {
    if (!await this.canNext()) return false;
    if (this._nextSummaryActive) return;
    let steps = this._getSteps(),
      self = this,
      active = this.state.activeStep,
      nextActive = active + 1,
      nextStep = this._steps[nextActive];
    while (nextStep && nextStep.props.disabled) {
      nextActive++;
      nextStep = this._steps[nextActive];
    }
    let nextSummary = this._summaries[nextActive];
    if (!nextSummary) return doUpdate();

    function doUpdate() {
      if (nextActive < steps.length) {
        let upd = {
          activeStep: nextActive
        };
        let eIdx = self.state.errors.indexOf(active);
        if (eIdx !== -1) {
          upd.errors = self.state.errors.concat([]);
          upd.errors.splice(eIdx, 1);
        }
        if (self.state.completed.indexOf(active) === -1) {
          upd.completed = self.state.completed.concat([]);
          upd.completed.push(active);
        }
        self.setState(upd, () => {
          self.props.onChange && self.props.onChange(self._getStepId(self.state.activeStep));
        });
        return true;
      } else {
        let upd = {
          completed: self.state.completed.concat([])
        };
        if (self.state.completed.indexOf(nextActive - 1) === -1) {
          upd.completed.push(nextActive - 1);
        }
        self.setState(upd, () => {
          self.props.onSave && self.props.onSave();
        });
        return true;
      }
    }

    this._nextSummaryActive = true;
    nextSummary.show((ok) => {
      this._nextSummaryActive = false;
      if (!ok) return;
      doUpdate();
    });

  }

  /**
   * Tries to go backwards to the previous step
   * */
  back() {
    if (!this.canBack()) return false;
    let active = this.state.activeStep,
      nextActive = active - 1,
      prevStep = this._steps[nextActive];
    while (prevStep && prevStep.props.disabled) {
      nextActive--;
      prevStep = this._steps[nextActive];
    }
    if (nextActive < 0) return false;
    let upd = {
      activeStep: nextActive,
      completed: this.state.completed.concat([])
    };
    let cIdx = this.state.completed.indexOf(nextActive);
    if (cIdx !== -1) {
      upd.completed.splice(cIdx, 1);
    }
    let finalIdx = upd.completed.indexOf(nextActive + 1);
    if (finalIdx !== -1) {
      upd.completed.splice(finalIdx, 1);
    }
    this.setState(upd, () => {
      this.props.onChange && this.props.onChange(this._getStepId(this.state.activeStep));
    });
    return true;
  }


  /********
   *
   * PRIVATE FUNCTIONALITY
   * */

  /*
  * Returns all Stepper.Content components, for raw children
  * */
  _getRawContent(children) {
    let content = [];
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (child.type !== Content) continue;
      content.push(child);
    }
    return content;
  }

  /*
  * Returns the body content of the active step
  * */
  _getContent(children) {
    let stepIdx = 0,
      toggle = this.props.toggle,
      activeStep = this.state.activeStep;
    if (!toggle) {
      let contents = [];
      for (let i = 0, len = children.length; i < len; i++) {
        let child = children[i];
        if (child.type !== Step) continue;
        contents.push(
          <div key={i} className={classNames('app-stepper-content-item', {
            'app-stepper-content-active': activeStep === stepIdx
          })}>
            {child.props.children}
          </div>
        );
        stepIdx++;
      }
      return contents;
    }
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (child.type !== Step) continue;
      if (activeStep === stepIdx) return child.props.children;
      stepIdx++;
    }
  }

  /*
  * Returns the "ID" of a step, or the step idx in stead
  * */
  _getStepId(step) {
    if (typeof this._stepMapIdx[step] === 'undefined') return step;
    return this._stepMapIdx[step];
  }

  /*
  * Returns the index of a given step id
  * */
  _getStepIdx(id) {
    if (typeof this._stepMap[id] !== 'undefined') return this._stepMap[id];
    return id;
  }

  /*
  * Returns the wizard actions
  * */
  _getActions(children, steps) {
    let actions = [];
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (child.type === Actions.Next) {
        let isDisabled = this.state.errors.indexOf(this.state.activeStep) !== -1;
        if (!isDisabled && this.state.isLoading) isDisabled = true;
        if (this.state.canNext === false) isDisabled = true;
        let opt = {
          key: i,
          disabled: isDisabled,
          onClick: () => {
            if (child.props.onClick) {
              let res = child.props.onClick(this._getStepId(this.state.activeStep));
              if (res === false) return;
              if (typeof res === 'undefined' || res === null) return this.next();
              if (typeof res === 'object' && typeof res.then === 'function' && typeof res.catch === 'function') {
                res.then(this.next);
              }
            } else {
              this.next();
            }
          }
        };
        if (this.state.activeStep + 1 === steps.length) {
          opt.label = child.props.labelSave;
          if (!opt.disabled) opt.disabled = !this.canSave();
        }
        actions.push(React.cloneElement(child, opt));
      } else if (child.type === Actions.Back) {
        let opt = {};
        if (child.props.disabled === false) {
          opt = Object.assign({}, child.props);
          opt.key = i;
        } else {
          opt = {
            key: i,
            disabled: !this.canBack(),
            onClick: () => {
              if (child.props.onBack) {
                let res = child.props.onClick(this._getStepId(this.state.activeStep));
                if (res === false) return;
                if (typeof res === 'undefined' || res === null) return this.back();
                if (typeof res === 'object' && typeof res.then === 'function' && typeof res.catch === 'function') {
                  res.then(this.back);
                }
              } else {
                this.back();
              }
            }
          };
        }
        actions.push(React.cloneElement(child, opt));
      }
    }
    return actions;
  }

  /*
  * Returns the array of steps
  * */
  _getSteps(children) {
    if (!children) children = this._getChildren();
    let items = [],
      stepIdx = 0;
    for (let i = 0, len = children.length; i < len; i++) {
      let childObj = children[i];
      if (childObj.type !== Step) continue;
      let props = childObj.props,
        isCompleted = typeof props.completed === 'boolean' ? props.completed : this.isCompleted(stepIdx),
        isError = this.isError(stepIdx),
        labelProps = {
          completed: isCompleted,
          disabled: props.disabled,
          className: classNames(props.className, 'app-stepper-label', {
            'app-stepper-label-error': isError,
            'app-stepper-label-completed': isCompleted
          })
        };
      if (isError) {
        labelProps.icon = props.errorIcon || <WarningIcon className="app-stepper-error-icon"/>;
      }
      if (props.id) {
        this._stepMapIdx[stepIdx] = props.id.toString();
        this._stepMap[props.id] = stepIdx;
      }
      items.push(
        <MuiStep
          key={i}
          completed={isCompleted}
          disabled={props.disabled}
        >
          <MuiStepLabel {...labelProps}>{props.label}</MuiStepLabel>
        </MuiStep>
      );
      stepIdx++;
    }
    this._steps = items;
    return items;
  }

  /*
  * Returns a flat array of all our children
  * */
  _getChildren(_children) {
    if (!_children) _children = this.props.children;
    let items = [],
      children = (_children instanceof Array ? _children : [_children]);
    for (let i = 0, len = children.length; i < len; i++) {
      let t = children[i];
      if (t instanceof Array) {
        for (let j = 0, jlen = t.length; j < jlen; j++) {
          let v = t[j];
          if (typeof v === 'object' && v && v.props) {
            items.push(v);
          }
        }
      } else if (typeof t === 'object' && t && t.props) {
        items.push(t);
      }
    }
    return items;
  }

  /*
  * Returns all our Summaries components
  * */
  _getSummaries(children) {
    let items = [],
      stepIdx = 0;
    for (let i = 0, len = children.length; i < len; i++) {
      let child = children[i];
      if (child.type === Summary) {
        let sumStepId = stepIdx;
        let clone = React.cloneElement(child, {
          key: i,
          onLoad: (c) => {
            if (c) {
              this._summaries[sumStepId] = c;
            } else {
              delete this._summaries[sumStepId];
            }
          }
        });
        items.push(clone);
      }
      if (child.type === Step) {
        stepIdx++;
      }
    }
    this.summaries = items;
    return items;
  }

  renderLoading() {
    if (!this.state.isLoading) return;
    return (
      <div className="app-stepper-loading">
        <CircularProgress size={30} thickness={2}/>
      </div>
    );
  }

  render() {
    const children = this._getChildren(),
      steps = this._getSteps(children),
      content = this._getContent(children),
      rawContent = this._getRawContent(children),
      actions = this._getActions(children, steps),
      summaries = this._getSummaries(children);
    if (!steps || steps.length === 0) return <div/>;
    return (
      <div
        ref={(c) => this.root = c}
        className={classNames("app-stepper", `app-stepper-size-${steps.length}`, {
          'app-stepper-toggle': !this.props.toggle
        }, this.props.className)}
        style={this.props.style}
      >
        {this.renderLoading()}
        {summaries}
        <div className="app-stepper-header">
          <MuiStepper
            linear={true}
            activeStep={this.state.activeStep}
            orientation={this.props.orientation}
          >
            {steps}
          </MuiStepper>
        </div>
        <div className="app-stepper-content">
          {content}
        </div>
        {actions.length > 0 && (
          <div className="app-stepper-actions">
            {actions}
          </div>
        )}
        {rawContent}
      </div>
    );
  }

}

AppStepper.defaultProps = {
  orientation: 'horizontal',
  toggle: true
};

AppStepper.PropTypes = {
  step: React.PropTypes.any, // The step id or the position
  toggle: React.PropTypes.bool, // If set to true, we will unmount/remount the content on step change.
  loading: React.PropTypes.bool,
  onChange: React.PropTypes.func,
  canNext: React.PropTypes.func,
  canBack: React.PropTypes.func,
  canSave: React.PropTypes.func,
  onSave: React.PropTypes.func,
  orientation: React.PropTypes.oneOf(['horizontal']),
};

module.exports = AppStepper;
